<?php
/*
 * @component TimeLine Marker 
 * @copyright Copyright (C) 2012 Moonsoft.  
 * @license GPL 3.0
 * This program is free software: you can redistribute it and/or modify it under the terms of the 
 * GNU General Public License as published by the Free Software Foundation, either version 3 of the License, 
 * or (at your option) any later version.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
 * See the GNU General Public License for more details.
 * See <http://www.gnu.org/licenses/>.
 * More info www.moonsoft.es gestion@moonsoft.es 
 */
defined('_JEXEC') or die('Direct Access to this location is not allowed.');
 
class ModTimelinemarkerHelper {
  protected function _buildQueryToday(){
     $query = "SELECT DATE(CURDATE()) as srt_date, 
     			DAYOFWEEK(CURDATE()) as day_of_week, 
     			TIME_FORMAT(CURTIME(),'%H:%i') as str_time";
     return $query;
  }
  
  protected function _buildQueryTimeDiff($holiday_from){  	
  	$query = "SELECT DATEDIFF('".$holiday_from."',curdate()) as diff";
     return $query;
  }
  
  protected function _buildQueryHolidayDiff($holiday_from,$holiday_to){  	
  	$query = "SELECT DATEDIFF('".$holiday_to."','".$holiday_from."') as diff";
     return $query;
  }
  
  protected function _buildQueryDateAdd($date_from, $dates_add){  	
  	$query = "SELECT ADDDATE('".$date_from."',".$dates_add." ) as nextday";
     return $query;
  }
  
  protected function _buildQueryTimeNextDiff($time_to){  	
  	$query = "SELECT TIMEDIFF('".$time_to."',TIMESTAMP(now())) as diff";
     return $query;
  }
  
  protected function getToday() {       
    $db = JFactory::getDBO();
    $query = $this->_buildQueryToday();
    $db->setQuery($query);
    return $db->loadObject();
  }  
  
  protected function getDateDiff($holiday_from) {       
    $db = JFactory::getDBO();
    $query = $this->_buildQueryTimeDiff($holiday_from);
    $db->setQuery($query);
    return $db->loadResult();
  }
  
  protected function getDateAdd($date_from,$dates_add) {       
    $db = JFactory::getDBO();
    $query = $this->_buildQueryDateAdd($date_from, $dates_add);
    $db->setQuery($query);
    return $db->loadResult();
  }
  
  protected function getHolidayDiff($holiday_from,$holiday_to) {       
    $db = JFactory::getDBO();
    $query = $this->_buildQueryHolidayDiff($holiday_from,$holiday_to);
    $db->setQuery($query);
    return $db->loadResult();
  }

  protected function getTimeDiff($time_to) {       
    $db = JFactory::getDBO();
    $query = $this->_buildQueryTimeNextDiff($time_to);
    $db->setQuery($query);
    return $db->loadResult();
  }
  
  protected function isHoliday($dateFrom, $dateTo, $today){
  	if (!isset($dateFrom)||!isset($dateTo)) return false;
  	if((strtotime($dateFrom) <= strtotime($today) &&
  		(strtotime($dateTo) >= strtotime($today)))){
  		return true;
  	}
  	return false;
  }
  
  protected function checkTime($timeFrom, $timeTo, $today){
  	if (!isset($timeFrom)||!isset($timeTo)) return false;
  	if((strtotime($timeFrom) <= strtotime($today) &&
  		(strtotime($timeTo) >= strtotime($today)))){
  		return true;
  	}
  	return false;
  }
  
  protected function getNextOfficeDay($date_off, $date_str, $days, $holiday_from, $holiday_to){
  	// Next day
  	$ret=array();
  	$dates_to_add=0;
  	$next_day=$date_off+1;
  	while (true){
  		if ($next_day>=8) $next_day=1;
  		if ($dates_to_add>366) {
  			$ret['date']=null;  		
  			$ret['date_num']=null; 
  			break;
  		}  		
  		if (!in_array($next_day, $days)){
  			$dates_to_add++;
  			$next_day++;
  			continue;
  		}
  		$new_date=$this->getDateAdd($date_str, $dates_to_add+1);
  		if ($holiday_from!=null && $holiday_to!=null &&
  			$this->isHoliday($holiday_from, $holiday_to, $new_date)){
  			$dates_to_add++;
  			$next_day++;
  			continue;
  		} 
  		$ret['date']=$new_date;  		
  		$ret['date_num']=$next_day;
  		break;
  	}
  	return $ret;
  } 
  
  protected function formatDate($date, $date_format){
  	if ($date_format!=null){
	  	$timestamp=strtotime($date);
		return date($date_format, $timestamp);
  	}
  	return $date;
  }
   
  public function getState($params){  	
  	$result=array();  	
  	if ($params->get('days')==null || !is_array($params->get('days'))){
  		$result['state']=-1;
  		return $result;
  	}  	  	
  	$data=$this->getToday();
  	// Holiday Check
  	// Date Check
  	// Hour Check
  	$is_holiday=$this->isHoliday($params->get('holidays_f'),$params->get('holidays_t'),$data->srt_date);
  	if ($is_holiday) $result['state']=2;
  	elseif (!in_array($data->day_of_week, $params->get('days'))) $result['state']=1;
  	else {  	
  		$_isOntime=$this->checkTime($params->get('hours_'.$data->day_of_week.'_f'), $params->get('hours_'.$data->day_of_week.'_t'), $data->str_time);
  		if ($_isOntime) $result['state']=0;
  		else $result['state']=1;
  	}
  	// Time to next holidays
  	if (($params->get('holidays_f')!=null || $params->get('holidays_t')!=null) && !$is_holiday ){
  		$days_to_holidays=$this->getDateDiff($params->get('holidays_f'));
  		if($days_to_holidays>0){ 
  			$result['days_to_holidays']=$days_to_holidays;	  		
	  		// Holidays
  			$result['number_holiday_days']=$this->getHolidayDiff($params->get('holidays_f'),$params->get('holidays_t'));
  		} 
  	}
  	if ($result['state']>0){
  		$next_day_office=null;
  		$next_day_office_date=null;
  		// Calculate next open day/hour
  		$_isOntime=$this->checkTime('00:01', $params->get('hours_'.$data->day_of_week.'_f'), $data->str_time);
  		if ($_isOntime){
  			$next_day_office=$this->getTimeDiff($data->srt_date.' '.$params->get('hours_'.$data->day_of_week.'_f').':00');	
  			$next_day_office_date=$data->srt_date;
  		}else{  		
	  		// Next day open
	  		$next_office_date=$this->getNextOfficeDay($data->day_of_week, $data->srt_date, $params->get('days'), $params->get('holidays_f'),$params->get('holidays_t'));
	  		if ($next_office_date['date']!=null){
	  			$next_day_office=$this->getTimeDiff($next_office_date['date'].' '.$params->get('hours_'.$next_office_date['date_num'].'_f').':00');
	  			$next_day_office_date=$next_office_date['date'];  			
	  		}
  		}
  		// Result
  		$arr_vals_office=explode(':', $next_day_office);
  		$days=0;
  		$hours=$arr_vals_office[0];
  		$minutes=$arr_vals_office[1];
  		if ($arr_vals_office[0]>=24){
  			$days=intval($arr_vals_office[0]/24);
  			$hours=$arr_vals_office[0]-(24*$days);  					 
  		}
  		$result['next_office_in']="";
      if($days>0)
      $result['next_office_in'].=$days.' '.($days==1?JText::_('TLM_DAY'):JText::_('TLM_DAYS')).' ';
      if($hours>0)
      $result['next_office_in'].=$hours.' '.($hours==1?JText::_('TLM_HOUR'):JText::_('TLM_HOURS')).' ';
      if($minutes>0)
  		$result['next_office_in'].=$minutes.' '.($minutes==1?JText::_('TLM_MINUTE'):JText::_('TLM_MINUTES'));
  		$result['next_office_in_date']=$this->formatDate($next_day_office_date,$params->get('date_format'));
  	}
  	
  	if ($this->checkTime($params->get('notice'), $params->get('holidays_f'), $data->str_time)){
  		$result['notice_holiday_date']=$this->formatDate($params->get('holidays_f'),$params->get('date_format'));
  		$result['number_holiday_to']=$this->formatDate($params->get('holidays_t'),$params->get('date_format'));
  	}  	
  	return $result;
  }  
  

  
} 
?>
